package Stanford::GCP::Storage;

use strict;
use warnings;
use autodie;

use Stanford::GCP::Bucket ;
use Stanford::GCP::Bucket::Object ;
use MIME::Base64;

use base qw( Exporter ) ;
our @EXPORT_OK = qw
  (
   gsutil_long_to_perl
   get_latest_info
  ) ;

our $VERSION = '0.09' ;


# Convert the output of "gsutil ls -L" to a Perl structure.
#
# {
#    'path' => 'gs:://xxxxxx',
#    'mtime_epoch' => 1556294905,
# }
sub gsutil_long_to_perl {
    my ($str) = @_ ;

    my @results = () ;

    my $bucket ;

    #### # # ###     #### # # ###     #### # # ###     #### # # ###
    my $add_results_fref = sub {
        my ($current_href) = @_ ;
        if (exists($current_href->{'path'})) {
            my %tmp = %{ $current_href } ;
            my $obj = Stanford::GCP::Bucket::Object->new() ;
            $obj->bucket($bucket) ;
            $obj->path($current_href->{'path'}) ;
            $obj->mtime_epoch($current_href->{'mtime_epoch'}) ;
            $obj->content_length($current_href->{'content_length'}) ;
            $obj->md5sum_ascii($current_href->{'md5sum_ascii'}) ;
            $obj->md5sum_base64($current_href->{'md5sum_base64'}) ;
            push(@results, $obj) ;
        } ;
    } ;
    #### # # ###     #### # # ###     #### # # ###     #### # # ###

    my %current = () ;
    my ($path, $mtime_epoch) ;
    my @lines = split(/\n/, $str) ;
    foreach my $line (@lines) {
        if ($line =~ m{^gs://([^\/]+)\/}xsm) {
            my $bucket_name = $1 ;
            if (!$bucket) {
                $bucket = Stanford::GCP::Bucket->new() ;
                $bucket->name($bucket_name) ;
            }
            $current{'bucket_name'}  = $1 ;
            # This is a first line, so save current stuff.
            $add_results_fref->(\%current) ;
            %current = () ;
            # Remove any trailing ':'
            $line =~ s{:$}{} ;

            $current{'path'} = $line ;
        } elsif ($line =~ m{^\s+goog-reserved-file-mtime:(\d+)\s*$}xsm) {
            $current{'mtime_epoch'} = $1 ;
        } elsif ($line =~ m{^\s+Content-Length:\s+(\d+)\s*$}xsm) {
            $current{'content_length'} = $1 ;
        } elsif ($line =~ m{^\s+Hash\ \(md5\):\s*(\S+)$}ixsm) {
            my $md5hash_b64    = $1;
            my $md5hash_binary = decode_base64($md5hash_b64) ;
            my @md5hash_text   =  unpack '(H2)*', $md5hash_binary  ;
            my $md5hash_text   =  join(q{}, @md5hash_text) ;
            $current{'md5sum_ascii'}  = $md5hash_text ;
            $current{'md5sum_base64'} = $md5hash_b64 ;
        } else {
            # Do nothing
        }
    }
    $add_results_fref->(\%current) ;

    return @results ;
}

# Given the output of a "gsutil ls -L" return the age in seconds and the
# size in bytes of the latest entry.
sub get_latest_info {
    my ($gs_long_list_string, $verbose) = @_ ;

    # Convert to a set of bucket objects.
    my @bucket_objects = gsutil_long_to_perl($gs_long_list_string) ;

    # Sort results by last-modified time
    my @buckets_by_mtime = sort {$a->mtime_epoch() cmp $b->mtime_epoch()} @bucket_objects;

    my $latest_bucket_object = $buckets_by_mtime[-1] ;
    if ($verbose) {
        print $latest_bucket_object->to_string() ;
        print "\n" ;
    }

    my $latest_backup_epoch = $latest_bucket_object->mtime_epoch() ;
    my $latest_backup_mtime = $latest_bucket_object->mtime() ;
    my $latest_backup_cl    = $latest_bucket_object->content_length() ;

    my $current_time_epoch  = time() ;

    my $diff_seconds = $current_time_epoch - $latest_backup_epoch ;
    my $diff_hours   = $diff_seconds / (60 * 60) ;

    return ($diff_seconds, $latest_backup_cl + 0) ;
}

1;
